import { DestFns, SrcFn, load_settings, get_settings_file } from './settings'
import fetch, { Headers } from 'node-fetch'
import { deploy_source, deploy_destination } from './deployer'

export async function deploy(access_token?: string, work_slug?: string, work_id?: string) {
  let settings = await load_settings(get_settings_file())
  if (settings.fn_type == 'source') {
    await deploy_source(settings, access_token, work_slug, work_id)
  } else {
    await deploy_destination(settings, access_token, work_slug, work_id)
  }
}

export type SrcPayload = {
  body: string | object
  headers: { [key: string]: string },
  queryParameters: { [key: string]: string | number | boolean }
}

function urlArgsToString(urlArgs: { [key: string]: string | number | boolean }) {
  if (!urlArgs) {
    return ''
  }
  let keys = Object.keys(urlArgs);
  if (keys.length == 0) {
    return ''
  }
  let s = '?'
  keys.forEach(key => {
    let value = urlArgs[key]
    let value_str = value
    s += '&' + key + '=' + value_str
  })
  return s
}

export function test_src(payload: SrcPayload, sam_port: number, sam_host?: string) {
  if (typeof payload.body !== 'string') {
    payload.headers['Content-Type'] = 'application/json'
  }
  return fetch(`http://${sam_host || '127.0.0.1'}:${sam_port}/function${urlArgsToString(payload.queryParameters)}`, {
    method: 'post',
    body: typeof payload.body == 'string' ? payload.body : JSON.stringify(payload.body),
    headers: payload.headers
  }).then(r => r.json())
}

export async function test_dest(payload: SegmentTrackEvent | SegmentGroupEvent | SegmentIdentifyEvent | SegmentAliasEvent | SegmentScreenEvent | SegmentPageEvent, sam_port: number, sam_host?: string) {
  return fetch(`http://${sam_host || '127.0.0.1'}:${sam_port}/function`, {
    method: 'post',
    body: JSON.stringify(payload),
    headers: {
      'Content-Type': 'application/json'
    }
  })
}

export async function call_src(payload: any, onRequest: SrcFn, output: any) {
  output.tracks = []
  output.identifies = []
  output.groups = []
  output.aliases = []
  output.screens = []
  output.pages = [];

  (global as any).Segment = {
    identify(i: SegmentOptions & SegmentSourceIdentify & SegmentId) {
      output.identifies.push(i)
    },
    track<E extends SegmentEvents>(t: SegmentOptions & SegmentTrackObject<E> & SegmentId) {
      output.tracks.push(t)
    },
    group(t: SegmentOptions & SegmentGroup & SegmentId) {
      output.groups.push(t)
    },
    alias(t: SegmentOptions & SegmentAlias & SegmentId) {
      output.aliases.push(t)
    },
    page(t: SegmentOptions & SegmentPage & SegmentId) {
      output.pages.push(t)
    },
    screen(t: SegmentOptions & SegmentScreen & SegmentId) {
      output.screens.push(t)
    }
  };


  let body = payload.body
  const settings = {}

  onRequest({
    json() {
      return JSON.parse(body)
    },
    text() {
      return body
    },
    headers: new Headers(payload.headers) as any,
    // TODO: This mimics the Segment Test approach. I suspect no one uses this in reality, but does this need to match the actual URL called by the test?
    url: new URL('https://fn.segmentapis.com/' + urlArgsToString(payload.queryStringParameters))
  }, settings)
}

export async function call_dest(_event: any, fns: DestFns) {
  let event = JSON.parse(_event.body)
  const settings = {}
  switch (event.type) {
    case 'alias': {
      if (fns.onAlias) {
        fns.onAlias(event, settings)
      }
      break
    }
    case 'identify': {
      if (fns.onIdentify) {
        fns.onIdentify(event, settings)
      }
      break
    }
    case 'page': {
      if (fns.onPage) {
        fns.onPage(event, settings)
      }
      break
    }
    case 'screen': {
      if (fns.onScreen) {
        fns.onScreen(event, settings)
      }
      break
    }
    case 'track': {
      if (fns.onTrack) {
        fns.onTrack(event, settings)
      }
      break
    }
    case 'group': {
      if (fns.onGroup) {
        fns.onGroup(event, settings)
      }
      break
    }
  }
}